﻿//+--------------------------------------------------------------------------+
//|                                                                          |
//|                               IconMenu                                   |
//|                         GDImage 3D animation                             |
//|                                                                          |
//|          The GDImage.dll trial version has no time limitation,           |
//|            and all features of the API are fully functional.             |
//|                                                                          |
//|      Buying a commercial license removes the copyright nag screen.       |
//|                                                                          |
//| L'utilisation à titre privé de la version d'évaluation de GDImage n'est  |
//|   pas limitée dans le temps et toutes les fonctions sont disponibles.    |
//|                                                                          |
//|              L'acquisition d'une licence permet de supprimer             |
//|                       l'affichage du copyright.                          |
//|                                                                          |
//|                             Version 1.00                                 |
//+--------------------------------------------------------------------------+
//|                                                                          |
//|                         Author Patrice TERRIER                           |
//|                           copyright (c) 2007                             |
//|                                                                          |
//|                        pterrier@zapsolution.com                          |
//|                                                                          |
//|                          www.zapsolution.com                             |
//|                                                                          |
//+--------------------------------------------------------------------------+
//|                  Project started on : 03-31-2007 (MM-DD-YYYY)            |
//|                        Last revised : 04-01-2007 (MM-DD-YYYY)            |
//+--------------------------------------------------------------------------+

using System;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Threading;

using Win32;
using SkinEngine;
using GDImage;

namespace IconMenu
{
    [UnmanagedFunctionPointer(CallingConvention.StdCall)]
    public delegate bool CallBack(IntPtr hWnd, int wMsg, int wParam, int lParam);

    public partial class MAIN_Form : Form
    {
        const int TIMER_MAX = 2;                         // To perform smooth animation
        const int ID_CTRL = -1;                          // Main GDImage control
        const int ID_THUMB = -2;                         // Thumb control

        const int ID_OBJECT_THUMB = 7;
        const int ID_OBJECT_TEXT = 8;                    // Each of the sprite must use a unique ID
        const int ID_OBJECT_LABEL = 9;
	    const int ID_OBJECT_SPRITE	= 10;
	
//	    const double PiDiv180 = 1.74532925199433E-2;

        static public IntPtr hFORM_Main = IntPtr.Zero;    // Default to Zero
        static public IntPtr hGDImageCtrl = IntPtr.Zero;  // Default to Zero
        static public IntPtr hGDImageThumb = IntPtr.Zero; // Default to Zero
        static public IntPtr hBackGround = IntPtr.Zero;   // Default to Zero

        public struct RECT { public int left, top, right, bottom;}

        public struct SpriteDataStruct
        {
            public string ImageName;
            public IntPtr hBitmap;
	        public int ID;
	        public int xPos;
	        public int yPos;
	        public float scale;
        }

        static SpriteDataStruct[] SpriteData = new SpriteDataStruct[5];
        static int gnMarqueeWidth, gnMarqueeHeight;

        static float grScaleDefault = 0.75f;
        static float grScaleStep = 0.01f;
        static int gnInflateID = 0;
        static int gnSelectID = 0;
        static bool gbSelectID = false;
        static bool gbAnimate = false;

        // Very important the call back must be static
        // to prevent from access violation during garbage collection.
        // Search for <<Callback Functions>>
        static CallBack UseCallBack = null;

        public MAIN_Form()
        {

            // We use a callback to handle the GDImage events         
            // Very important the call back must be static
            // to prevent from access violation during garbage collection.
            // Search for <<Callback Functions>>
            UseCallBack = new CallBack(GDImageCallBack);
           
            hFORM_Main = this.Handle;

            this.SetStyle(ControlStyles.DoubleBuffer, true);
            this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            this.SetStyle(ControlStyles.UserPaint, true);
            this.SetStyle(ControlStyles.ResizeRedraw, false);
            //this.SetStyle(ControlStyles.SupportsTransparentBackColor, true);
            ////this.SetStyle(ControlStyles.StandardDoubleClick, false);
            ////this.SetStyle(ControlStyles.Selectable, true);
            //this.ResizeRedraw = true;

            // Setup Skin parameters
            SetSkinParameters();

            InitializeComponent();

            GDImage_Template.Visible = false;
            GDImage_Template.Enabled = false;
            //SaveInitialValue();

            // Create the GDImage control using SDK style
            uint Style = Api.WS_CHILD | Api.WS_VISIBLE;
            uint StyleEx = 0;
            hGDImageCtrl = Api.CreateWindowEx(StyleEx,
                           GI.GDImageClassName,             // window class name
                           "",                              // window title
                           Style,                           // window style
                           GDImage_Template.Location.X,     // x location
                           GDImage_Template.Location.Y,     // y location
                           GDImage_Template.Size.Width,     // Width
                           GDImage_Template.Size.Height,    // Height
                           GDImage_Template.Parent.Handle,  // parent window handle
                           ID_CTRL,                         // ControlID
                           Api.GetModuleHandle(""),         // program instance handle
                           0);                              // creation parameters

            if (hGDImageCtrl != IntPtr.Zero) {
                GI.SetProperty(hGDImageCtrl, GI.ZI_GradientTop, Api.RGB(27, 29, 41));
                GI.SetProperty(hGDImageCtrl, GI.ZI_GradientBottom, Api.RGB(27, 29, 41));

                hGDImageThumb = Api.CreateWindowEx(StyleEx,
                                GI.GDImageClassName,          // window class name
                                "",                           // window title
                                Style,                        // window style
                                GDImage_Thumb.Location.X,     // x location
                                GDImage_Thumb.Location.Y,     // y location
                                GDImage_Thumb.Size.Width,     // Width
                                GDImage_Thumb.Size.Height,    // Height
                                GDImage_Thumb.Parent.Handle,  // parent window handle
                                ID_THUMB,                     // ControlID
                                Api.GetModuleHandle(""),      // program instance handle
                                0);                           // creation parameters

                if (hGDImageThumb != IntPtr.Zero) {
                    GI.SetProperty(hGDImageThumb, GI.ZI_GradientTop, Api.RGB(27, 29, 41));
                    GI.SetProperty(hGDImageThumb, GI.ZI_GradientBottom, Api.RGB(27, 29, 41));
                    int x, y, bmW, bmH;
                    IntPtr hBitmap = GI.ZI_CreateBitmapFromFile(Api.ExePath() + "user.png", out bmW, out bmH);
                    if (hBitmap != IntPtr.Zero){
                        x = (GDImage_Thumb.Width - bmW) / 2;
                        y = (GDImage_Thumb.Height - bmH) / 2;
                        GI.ZD_DrawBitmapToCtrl(hGDImageThumb, x, y, hBitmap, GI.ZD_ColorARGB(0, Api.RGB(255, 255, 255)), ID_OBJECT_THUMB, GI.ZS_VISIBLE);
                        GI.ZD_UsePngOpacity(ID_OBJECT_THUMB, true);
                        GI.ZD_SetObjectLocked(ID_OBJECT_THUMB, true);
                    }
                }

                LoadSprites();
                StartAnimation();
                gbAnimate = true; // Enable animation
            }

        }

        private void SetSkinParameters()
        {
            // REM this out to disable the drop shadow effect.
            //Api.UseDropShadow(hFORM_Main);

            // Setup default Skin colors
            SK.ColorLayerBackground = Color.FromArgb(192, 192, 202);
            SK.ColorCaptionEnabled = Color.FromArgb(134, 153, 190);
            SK.ColorCaptionDisabled = Color.FromArgb(73, 76, 99);
            SK.TooltipBackColor = Color.FromArgb(73, 76, 99);
            SK.TooltipForeColor = Color.FromArgb(134, 153, 190);

            // Skin opacity in the range 0-255
            SK.Alpha = 255;//204; // 80 % default transparency level

            // Skin opacity of the hidden background in the range 0-254
            //SK.OpacityLayerBackground = 248;

            //SK.IsResizable = false;
        }

        private void MonitorGDImageEvents()
        {
            UseCallBack = new CallBack(GDImageCallBack);

            // We use a callback to monitor the GDImage control messages
            // Create a %WM_LBUTTONDOWN event
            GI.ZI_EventMessage(GDImageCallBack, Api.WM_LBUTTONDOWN, true);
            // Create a WM_LBUTTONDBLCLK event
            GI.ZI_EventMessage(GDImageCallBack, Api.WM_LBUTTONDBLCLK, true);
            // Create a WM_RBUTTONDOWN event
            GI.ZI_EventMessage(GDImageCallBack, Api.WM_RBUTTONDOWN, true);
            // Create a WM_KEYDOWN event
            GI.ZI_EventMessage(GDImageCallBack, Api.WM_KEYDOWN, true);
            // Create a WM_MOUSEMOVE event
            GI.ZI_EventMessage(GDImageCallBack, Api.WM_MOUSEMOVE, true);

            GC.KeepAlive(UseCallBack);
        }

        private void LoadSprites()
        {
            string ExePath = Api.ExePath();
            SpriteData[0].ImageName = ExePath + "Office Publisher.png";
            SpriteData[1].ImageName = ExePath + "Office Word.png";
            SpriteData[2].ImageName = ExePath + "Office Outlook.png";
            SpriteData[3].ImageName = ExePath + "Office PowerPoint.png";
            SpriteData[4].ImageName = ExePath + "Office Excel.png";

            int UboundSpriteData = SpriteData.Length;
            int bmW, bmH, xLeft, yOffset, Item;
            xLeft = (GDImage_Template.Width - ((32 * (UboundSpriteData - 1)) + (64 * UboundSpriteData))) / 2;
            xLeft = xLeft - ((128 - 64) / 2);
            for (int K = 0; K < UboundSpriteData; K++)
            {
                Item = ID_OBJECT_SPRITE + K + 1;

                if (SWITCH_Mirror.SwitchStatus)
                {
                    SpriteData[K].hBitmap = GI.ZI_CreateMirrorBitmapFromFile(SpriteData[K].ImageName, out bmW, out bmH);
                    yOffset = 8; }
                else {
                    SpriteData[K].hBitmap = GI.ZI_CreateBitmapFromFile(SpriteData[K].ImageName, out bmW, out bmH);
                    yOffset = 0;
                }

                SpriteData[K].ID = Item;
                SpriteData[K].xPos = xLeft;
                SpriteData[K].yPos = 64;
	            xLeft += (64 + 32);

                GI.ZD_DrawBitmapToCtrl(hGDImageCtrl,
                                       SpriteData[K].xPos,
                                       SpriteData[K].yPos - yOffset,
                                       SpriteData[K].hBitmap,
                                       GI.ZD_ColorARGB(255, Api.RGB(255, 255, 255)),
                                       SpriteData[K].ID,
                                       GI.ZS_VISIBLE);

                SpriteData[K].scale = grScaleDefault;
                GI.ZD_SetObjectScale(SpriteData[K].ID, SpriteData[K].scale);

                GI.ZD_UsePngOpacity(SpriteData[K].ID, true);

                string sLabel = SpriteData[K].ImageName.Replace(ExePath, "");
                sLabel = sLabel.Replace(".png", "");
                GI.ZD_SetObjectImageLabel(SpriteData[K].ID, sLabel);

            }
            uint ARGB = GI.ZD_ColorARGB(128, Api.RGB(255, 255, 255));
            string MarqueeMsg = "Left mouse button down to drag the icon, double click to perform selection ...";
            string FontToUse = "Times New Roman";
            GI.ZD_DrawTextToCtrl(hGDImageCtrl,
            MarqueeMsg, 					// The text to be displayed
            GDImage_Template.Size.Width, 	// X coordinate
            2, 	    						// Y coordinate
            ARGB, 			    			// ARGB color to use
            FontToUse, 			    		// The True Type Font to use (must be a valid one)
            20, 							// The font size in pixel
            ID_OBJECT_TEXT, 				// The unique object ID
            GI.ZS_HIDDEN, 					// Overlay visible at startup
            0, 						    	// Optional shadow effect (offset in pixel)
            0);                             // Optional string format

            GI.ZD_GetObjectBound(ID_OBJECT_TEXT, out gnMarqueeWidth, out gnMarqueeHeight);

            // To display the label matching a specific icon (during mouse hover)
            ARGB = GI.ZD_ColorARGB(255, Api.RGB(255, 255, 255));
            FontToUse = "Times New Roman";
            GI.ZD_DrawTextToCtrl(hGDImageCtrl,
            "", 		        			// The text to be displayed
            GDImage_Template.Width,         // X coordinate
            -100,     						// Y coordinate
            ARGB, 			    			// ARGB color to use
            FontToUse, 			    		// The True Type Font to use (must be a valid one)
            40, 							// The font size in pixel
            ID_OBJECT_LABEL, 				// The unique object ID
            GI.ZS_HIDDEN, 					// Overlay visible at startup
            0, 						    	// Optional shadow effect (offset in pixel)
            0);                             // Optional string format

        }

        private void FORM_Load(object sender, EventArgs e)
        {
            // We need this to retrieve bitmap from resource
            SK.UseNameSpace = this.GetType().Namespace;

            // Use bitmap resources to setup public value for:
            // FixCaptionLeft, FixCaptionHeight, FixCaptionRight, FixBottomHeight.
            Bitmap bmp;
            bmp = Properties.Resources.TopLeft;
            SK.FixCaptionLeft = bmp.Width; SK.FixCaptionHeight = bmp.Height;
            bmp = Properties.Resources.BottomRight;
            SK.FixCaptionRight = bmp.Width; SK.FixBottomHeight = bmp.Height;
            SK.DockMin = SK.FixCaptionHeight + SK.FixBottomHeight; // Docking

            // Setup the minimum size of the form
            int UseMinWidth = Math.Max(this.MinimumSize.Width, SK.FixCaptionHeight + SK.FixBottomHeight);
            int UseMinHeight = Math.Max(this.MinimumSize.Height, SK.FixCaptionHeight + SK.FixBottomHeight);
            SK.UnDockingHeight = UseMinHeight; // Save the UnDockingHeight
            this.MinimumSize = new System.Drawing.Size(UseMinWidth, UseMinHeight);

            this.BackgroundImage = (Bitmap)SK.DrawBackGround(this.Width, this.Height);

            Bitmap bmpFrmBack = (Bitmap)this.BackgroundImage;
            SK.CreateFormRegion(this, bmpFrmBack);
            //RECT rw = new RECT();
            Api.RECT rw = new Api.RECT();
            Api.GetWindowRect(hFORM_Main, ref rw);
            SK.LastRgnWidth = rw.right - rw.left; SK.LastRgnHeight = rw.bottom - rw.top;

            // Status text color
            //STATUS_Label.ForeColor = SK.ColorCaptionEnabled;

            // Caption text + color
            if (this.Enabled)
            { CAPTION_Label.ForeColor = SK.ColorCaptionEnabled; }
            else
            { CAPTION_Label.ForeColor = SK.ColorCaptionDisabled; }
            CAPTION_Label.TextAlign = ContentAlignment.MiddleLeft;
            CAPTION_Label.Text = this.Text;

            // FORM_Tooltip colors
            FORM_Tooltip.BackColor = SK.TooltipBackColor;
            FORM_Tooltip.ForeColor = SK.TooltipForeColor;

            // Check first if we can use layered window.
            byte Alpha = 0;
            if ((Api.GetOsVersion() > 499) && (SK.Alpha < 255))
            {
                Alpha = 254; this.Opacity = (double)Alpha / 255;
            }
            else
            {
                Alpha = 255; this.Opacity = (double)Alpha / 255;
            }
            // Support for layered window is a prerequisite to create aero glass effect
            if (Alpha < 255)
            {
                Api.SetLayeredWindowAttributes(hFORM_Main, 0, SK.Alpha, 0x00000002);
            //    //Create the skin background
            //    Form BackGround = new Form();
            //    BackGround.ShowInTaskbar = false; // WS_EX_TOOLWINDOW
            //    BackGround.Visible = true;
            //    BackGround.FormBorderStyle = 0;
            //    BackGround.BackColor = SK.ColorLayerBackground;
            //    BackGround.Opacity = 0.999; // Don't change this !
            //    hBackGround = BackGround.Handle;
            //    Api.SetLayeredWindowAttributes(hBackGround, 0, SK.OpacityLayerBackground, 0x00000002);
            //    ResizeMoveHiddenWindows();
            }

        }

        // ****************************************
        // Callback that handles the GDImage events
        // Search for <<Callback Functions>>
        public unsafe static bool GDImageCallBack(IntPtr hWnd, int wMsg, int wParam, int lParam)
        {
            Boolean bRet = false; // Do not stop the event processing in GDImage
            //string sMessage;
            int ObjectID = 0;
            int x, y, x1, y1, useWidth, useHeight, BoundWidth, BoundHeight,
                x2Div2, y2Div2, UseStep;
            Api.RECT rc = new Api.RECT();

            if (hWnd == hGDImageCtrl)
            {
                ObjectID = GI.ZI_MouseOverObjectID();
                switch (wMsg)
                {
                    case Api.WM_LBUTTONDOWN:
                        break;

                    case Api.WM_LBUTTONDBLCLK:
                        if (ObjectID > ID_OBJECT_SPRITE)
                        {
                            GI.ZD_SetObjectAngle(ObjectID, 360, GI.ZD_DRAW_DEFERRED);
                            gnSelectID = ObjectID;
                            gbSelectID = true;

                            ////////////////////////////////////////////////
                            // Put you code there to perfom the selection //
                            ////////////////////////////////////////////////

                        }
                        break;

                    case Api.WM_RBUTTONDOWN:
                        switch (ObjectID)
                        {
                            case ID_CTRL:
                                break;
                            case ID_OBJECT_TEXT:
                                break;
                            case ID_OBJECT_SPRITE + 1:
                                break;
                            case ID_OBJECT_SPRITE + 2:
                                break;
                            case ID_OBJECT_SPRITE + 3:
                                break;
                            case ID_OBJECT_SPRITE + 4:
                                break;
                            case ID_OBJECT_SPRITE + 5:
                                break;
                            default:
                                break;
                        }
                        break;

                    case Api.WM_MOUSEMOVE:
                        if (ObjectID != 0) {
                            if (gnInflateID != ObjectID) {
                                gnInflateID = ObjectID;
                                if (GI.ZD_GetObjectZorder(ObjectID) != GI.ZD_ORDER_TOP) {
                                    GI.ZD_SetObjectZorder(ObjectID, GI.ZD_ORDER_TOP);

                                    Api.GetClientRect(Api.GetDlgItem(hFORM_Main, ID_CTRL), ref rc);
                                    
                                    // Show current label
                                    int nBoundWidth, nBoundHeight;
                                    GI.ZD_SetObjectARGBcolor(ID_OBJECT_LABEL, GI.ZD_ColorARGB(0, Api.RGB(255, 255, 255)));
                                    GI.ZD_SetObjectUse3Dshadow(ID_OBJECT_LABEL, 0); // Disable shadow if any
                                    string sLabel = GI.ZD_GetObjectImageLabel(ObjectID);
                                    GI.ZD_SetObjectText(ID_OBJECT_LABEL, sLabel);
                                    GI.ZD_GetObjectBound(ID_OBJECT_LABEL, out nBoundWidth, out nBoundHeight);
                                    GI.ZD_SetObjectXY(ID_OBJECT_LABEL, (rc.right - nBoundWidth) / 2, rc.bottom - (nBoundHeight * 2), GI.ZD_DRAW_DEFERRED);
                                    GI.ZD_SetObjectVisibility(ID_OBJECT_LABEL, GI.ZD_SHOW);
                                    GI.ZD_SetObjectZorder(ID_OBJECT_LABEL, GI.ZD_ORDER_BOTTOM);
                                    gbSelectID = false;
                                }
                            }
                        }
                        else  { // Clear the label
                            GI.ZD_SetObjectAlpha(ID_OBJECT_LABEL, (byte) 0, GI.ZD_DRAW_DEFERRED);
                            GI.ZD_SetObjectVisibility(ID_OBJECT_LABEL, GI.ZD_HIDE);
                            gbSelectID = false;
                        }
                        break;

                    case Api.WM_KEYDOWN:
                        ObjectID = GI.ZI_GetObjectFocusID();
                        if (ObjectID != 0)
                        {
                            GI.ZD_GetObjectXY(ObjectID, out x, out y);
                            x1 = x; // Make a copy to keep the orignal x location unchanged
                            y1 = y; // Make a copy to keep the orignal y location unchanged 

                            // Check accelerator keys to compute the step range 
                            if (GI.ZI_IsCtrlKeyPressed())
                            {
                                UseStep = 4;
                                if (GI.ZI_IsShiftKeyPressed()) UseStep = 16;
                            }
                            else
                            {
                                UseStep = 1;
                                if (GI.ZI_IsShiftKeyPressed()) UseStep = 2;
                            }

                            // If object scroll with the bitmap background
                            if (GI.ZD_GetObjectScroll(ObjectID))
                            {
                                // Get the size of the bitmap background
                                GI.ZI_GetBitmapSize(GI.ZI_GetBMP(Api.GetDlgItem(hFORM_Main, ID_CTRL)), out useWidth, out useHeight);
                            }
                            else
                            {
                                // Get the control client size  
                                Api.GetClientRect(Api.GetDlgItem(hFORM_Main, ID_CTRL), ref rc);
                                useWidth = rc.right; useHeight = rc.bottom;
                            }
                            // Get the sprite object size
                            GI.ZD_GetObjectBound(ObjectID, out BoundWidth, out BoundHeight);

                            x2Div2 = (int)(BoundWidth / 2); y2Div2 = (int)BoundHeight / 2;

                            switch (wParam)
                            {
                                case Api.VK_HOME:
                                    x1 = 0;
                                    break;
                                case Api.VK_END:
                                    x1 = Math.Max(useWidth - BoundWidth, 0);
                                    break;
                                case Api.VK_PRIOR:
                                    y1 = 0;
                                    break;
                                case Api.VK_NEXT:
                                    y1 = Math.Max(useHeight - BoundHeight, 0);
                                    break;
                                case Api.VK_NUMPAD4:
                                case Api.VK_LEFT:
                                    if (x1 > -x2Div2) x1 = Math.Max(x1 - UseStep, -x2Div2);
                                    break;
                                case Api.VK_NUMPAD8:
                                case Api.VK_UP:
                                    if (y1 > -y2Div2) y1 = Math.Max(y1 - UseStep, -y2Div2);
                                    break;
                                case Api.VK_NUMPAD6:
                                case Api.VK_RIGHT:
                                    if (x1 < useWidth - x2Div2)
                                    {
                                        x1 = Math.Min(x1 + UseStep, useWidth - x2Div2);
                                    }
                                    break;
                                case Api.VK_NUMPAD2:
                                case Api.VK_DOWN:
                                    if (y1 < useHeight - y2Div2)
                                    {
                                        y1 = Math.Min(y1 + UseStep, useHeight - y2Div2);
                                    }
                                    break;
                                default:
                                    break;
                            }

                            if ((x != x1) || (y != y1))
                            {
                                x = x1; y = y1;
                                GI.ZD_SetObjectXY(ObjectID, x1, y1, GI.ZD_DRAW_REDRAW);
                            }
                            //sMessage = "Object " + ObjectID.ToString() + " coordinates " + x.ToString() + "," + y.ToString();
                            //Api.SetWindowText(Api.GetDlgItem(hFORM_Main, ID_STATUS_Main), sMessage);
                            bRet = true;
                        }
                        break;
                }
            }
            return bRet;
        }

        private void FORM_Close(object sender, EventArgs e)
        {
            this.Dispose();
        }

        //***********************************************************
        // We use the .NET template to resize the GDImage control
        private void FORM_Main_Resize(object sender, EventArgs e)
        {
            Api.MoveWindow(hGDImageCtrl, GDImage_Template.Location.X, GDImage_Template.Location.Y, GDImage_Template.Size.Width, GDImage_Template.Size.Height, true);
        }

        private void FOCUS_Activated(object sender, EventArgs e)
        {
            if (Api.IsWindow(hBackGround))
            {
                uint swpStyle = Api.SWP_NOSIZE | Api.SWP_NOMOVE | Api.SWP_NOSENDCHANGING | Api.SWP_NOOWNERZORDER | Api.SWP_NOACTIVATE | Api.SWP_NOREDRAW;
                Api.SetWindowPos(hBackGround, IntPtr.Zero, 0, 0, 0, 0, swpStyle);
                swpStyle = Api.SWP_NOSIZE | Api.SWP_NOMOVE | Api.SWP_NOSENDCHANGING;
                Api.SetWindowPos(hFORM_Main, IntPtr.Zero, 0, 0, 0, 0, swpStyle);
            }
            if (LED_Off.Visible)
            {
                CAPTION_Label.ForeColor = SK.ColorCaptionEnabled;
                LED_On.Visible = true;
                LED_Off.Visible = false;
            }
        }

        private void SwitchOff()
        {
            CAPTION_Label.Select();
            CAPTION_Label.ForeColor = SK.ColorCaptionDisabled;
            LED_On.Visible = false;
            LED_Off.Visible = true;
        }

        private void FOCUS_Deactivate(object sender, EventArgs e)
        {
            SwitchOff();
        }

        protected override void WndProc(ref Message m)
        {

            // We monitor the GDImage events from there
            // And we are doing it from there to protect us from the infamous
            // CallbackOnCollectedDelegate
            // (rappel effectué sur un délégué par le garbage collector)
            MonitorGDImageEvents();

            switch (m.Msg)
            {
                case Api.WM_SIZE:

                    //Start flicker free section
                    StopAnimation();
//                  Api.ShowWindow(Skin_STATUS_Dark.Handle, Api.SW_HIDE);

                    SK.BuildRegion(m.HWnd);

                    //End flicker free section
//                  Api.ShowWindow(Skin_STATUS_Dark.Handle, Api.SW_SHOW);

                    // Resize GDImage controls to match their respective template location.
                    Api.MoveWindow(hGDImageCtrl, GDImage_Template.Location.X, GDImage_Template.Location.Y, GDImage_Template.Width, GDImage_Template.Height, true);
                    Api.MoveWindow(hGDImageThumb, GDImage_Thumb.Location.X, GDImage_Thumb.Location.Y, GDImage_Thumb.Width, GDImage_Thumb.Height, true);

                    StartAnimation();

                    m.Result = (IntPtr)0;
                    return;

                case Api.WM_NCLBUTTONDBLCLK:
                    // We handle this ourself to avoid havoc when using restore
                    // when using a form without caption.
                    if (SK.IsMaximized)
                    {
                        //Api.ButtonClick(BTN_Restore.Handle);
                        FORM_Restore();
                    }
                    else
                    {
                        //Api.ButtonClick(BTN_Maximize.Handle);
                        FORM_Maximize();
                    }
                    m.Result = (IntPtr)0;
                    return;

                case Api.WM_LBUTTONDBLCLK:
                    if (SK.IsMaximized)
                    { // We simulate double click on caption to restore the window size
                        if (Api.HiWrd((uint)m.LParam) < SK.FixCaptionHeight) Api.ButtonClick(BTN_Restore.Handle);
                    }
                    break;

                case Api.WM_ACTIVATEAPP: // Keep hidden background behind the form
                    if (Api.IsWindow(hBackGround))
                    {
                        if ((uint)m.WParam == 0)
                        {
                            uint swpStyle = Api.SWP_NOSIZE | Api.SWP_NOMOVE | Api.SWP_NOSENDCHANGING | Api.SWP_NOOWNERZORDER | Api.SWP_NOACTIVATE | Api.SWP_NOREDRAW;
                            Api.SetWindowPos(hBackGround, Api.GetWindow(hFORM_Main, 2), 0, 0, 0, 0, swpStyle);
                        }
                    }
                    break;

                case Api.WM_COMMAND:
                    // I am using IF there 
                    // because C# doesn't handle IntPtr in a SWITCH select case
                    if (m.LParam == SWITCH_Wallpaper.Handle)
                    { Wallpaper_Click(); }
                    else if (m.LParam == SWITCH_Mirror.Handle)
                    { 
                        int UboundSpriteData = SpriteData.Length;
                        int Item, x, y, bmW, bmH, yOffset;
                        for (int K = 0; K < UboundSpriteData; K++)
                        {
                            Item = ID_OBJECT_SPRITE + K + 1;
                            if (SWITCH_Mirror.SwitchStatus) {
                                SpriteData[K].hBitmap = GI.ZI_CreateMirrorBitmapFromFile(SpriteData[K].ImageName, out bmW, out bmH);
                                yOffset = -8;
                            }
                            else  {
                                SpriteData[K].hBitmap = GI.ZI_CreateBitmapFromFile(SpriteData[K].ImageName, out bmW, out bmH);
                                yOffset = 8;
                            }
                          	// Retrieve previous coordinates to redraw the sprite at the same location
                            GI.ZD_GetObjectXY(Item, out x, out y);
                            GI.ZD_DrawBitmapToCtrl(hGDImageCtrl,
                                                   x, y + yOffset,
                                                   SpriteData[K].hBitmap, 
                                                   GI.ZD_ColorARGB(255, 0), 
                                                   SpriteData[K].ID, 
                                                   GI.ZS_VISIBLE);
                        }

                    
                    }
                    else if (m.LParam == SWITCH_Marquee.Handle)
                    {
                        if (SWITCH_Marquee.SwitchStatus)
                        {
                            GI.ShowObject(ID_OBJECT_TEXT, GI.ZD_SHOW);
                        }
                        else
                        {
                            GI.ShowObject(ID_OBJECT_TEXT, GI.ZD_HIDE);
                        }
                    }
                    break;

                case Api.WM_TIMER:
                    int wParam = (int)m.WParam;
                    if (wParam == 1)
                    {
                        DrawSprite();
                    }
                    m.Result = (IntPtr)0;
                    return;

                case Api.WM_DESTROY:
                    StopAnimation();
                    break;
            }

            base.WndProc(ref m);

            // This is the place where the skin NC area is handled.
            if (m.Msg == Api.WM_NCHITTEST)
            {
                if (SK.IsMaximized) return;

                if (m.Result == (IntPtr)Api.HTCLIENT)
                {
                    int HITTEST = Api.HTCAPTION;
                    if (SK.IsResizable) // Check if the form is resizable
                    {
                        Point p = this.PointToClient(new Point(Api.LoWrd((uint)m.LParam), Api.HiWrd((uint)m.LParam)));

                        int xF = this.Width, yF = this.Height;
                        int xSide = 0;
                        int Border = System.Windows.Forms.SystemInformation.Border3DSize.Width;
                        if ((p.X >= xF - SK.FixBottomHeight) && ((p.Y >= yF - SK.FixBottomHeight)))
                        {
                            HITTEST = Api.HTBOTTOMRIGHT;
                        }
                        else
                        {
                            // Left side
                            if (p.X <= 8)
                            {
                                if (p.X <= Border) HITTEST = Api.HTLEFT;
                                xSide = 1;
                            }
                            // Right side
                            if (p.X >= xF - 8)
                            {
                                if (p.X >= xF - Border) HITTEST = Api.HTRIGHT;
                                xSide = 2;
                            }
                            // Top side
                            if (p.Y <= Border)
                            {
                                HITTEST = Api.HTTOP;
                                if (xSide == 1)
                                {
                                    HITTEST = Api.HTTOPLEFT;
                                }
                                else if (xSide == 2)
                                {
                                    HITTEST = Api.HTTOPRIGHT;
                                }
                            }
                            // Bottom side
                            if (p.Y >= yF - Border)
                            {
                                if (xSide == 1)
                                {
                                    HITTEST = Api.HTBOTTOMLEFT;
                                }
                                else
                                {
                                    HITTEST = Api.HTBOTTOM;
                                }
                            }
                        }
                    }

                    m.Result = (IntPtr)HITTEST;
                }
            }
        }

        private void BTN_Close_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void BTN_Iconize_Click(object sender, EventArgs e)
        {
            this.WindowState = FormWindowState.Minimized;
        }

        private int WasX, WasY, WasWidth, WasHeight;
        private void FORM_Restore()
        {
            //CLIENT_Panel.Visible = false;

            BTN_Maximize.Visible = true;
            BTN_Restore.Visible = false;

            bool RedrawMode = true;
            if (SK.Alpha < 255)
            {
                Api.SetLayeredWindowAttributes(hFORM_Main, 0, SK.Alpha, 0x00000002);
                RedrawMode = false;
                if (Api.IsWindow(hBackGround)) Api.MoveWindow(hBackGround, WasX, WasY, WasWidth, WasHeight, false);
            }
            Api.MoveWindow(hFORM_Main, WasX, WasY, WasWidth, WasHeight, RedrawMode);

            SK.IsMaximized = false;
        }
        private void BTN_Restore_Click(object sender, EventArgs e)
        {
            FORM_Restore();
        }

        private void FORM_Maximize()
        {
            WasX = this.Location.X;
            WasY = this.Location.Y;
            WasWidth = this.Width;
            WasHeight = this.Height;

            SK.IsMaximized = true;

            Api.ControlDrawingDisable(this.Handle);

            bool RedrawMode = true;
            if (SK.Alpha < 255)
            {
                RedrawMode = false;
                // Turn it to opaque mode
                Api.SetLayeredWindowAttributes(hFORM_Main, 0, 255, 0x00000002);
                if (Api.IsWindow(hBackGround)) Api.MoveWindow(hBackGround, -9990, -9990, WasWidth, WasHeight, false);
            }

            Api.MoveWindow(hFORM_Main, 0, 0, SystemInformation.WorkingArea.Width, SystemInformation.WorkingArea.Height, RedrawMode);

            BTN_Restore.Visible = true;
            BTN_Maximize.Visible = false;

            Api.ControlDrawingEnable(this.Handle);
        }
        private void BTN_Maximize_Click(object sender, EventArgs e)
        {
            FORM_Maximize();
        }

        private void StartAnimation()
        {
            // Use API WM_TIMER to avoid aggressive garbage collection 
            // to occur before Main ends.
            Api.SetTimer(hFORM_Main, 1, 0, null);
        }

        private void StopAnimation()
        {
            Api.KillTimer(hFORM_Main, 1);
        }

        private void DrawSprite()
        {
            // Based on 
            if (gbAnimate)
            {
                byte Alpha, Red, Green, Blue;
                
                int UboundSpriteData = SpriteData.Length;
                int Angle, K;
        
                for (K = UboundSpriteData - 1; K > -1; K--)
                {
                    if (gnInflateID == SpriteData[K].ID)
                    {
                        if (SpriteData[K].scale < 1) SpriteData[K].scale += grScaleStep;
                        GI.ZD_GetObjectARGB(ID_OBJECT_LABEL, out Alpha, out Red, out Green, out Blue);
                        if (Alpha < 255) {
                            Alpha += 3; GI.ZD_SetObjectAlpha(ID_OBJECT_LABEL, Alpha, 0);
                            if (Alpha == 255) {
                                GI.ZD_SetObjectARGBcolor(ID_OBJECT_LABEL, GI.ZD_ColorARGB(Alpha, Api.RGB(233, 233, 75)));
                                GI.ZD_SetObjectUse3Dshadow(ID_OBJECT_LABEL, 1); // Enable shadow
                            }
                        }
                    }
                    else {
                        if (SpriteData[K].scale > grScaleDefault) SpriteData[K].scale -= grScaleStep;
                    }
            		GI.ZD_SetObjectScale(SpriteData[K].ID, SpriteData[K].scale);
                    Angle = GI.ZD_GetObjectAngle(SpriteData[K].ID);
		            if (Angle > 0)
                    {
			            Angle -= 8; if (Angle < 0) Angle = 0;
                        GI.ZD_SetObjectAngle(SpriteData[K].ID, Angle, GI.ZD_DRAW_DEFERRED);
                    }
                }

                GI.ZD_GetObjectARGB(ID_OBJECT_THUMB, out Alpha, out Red, out Green, out Blue);
                if (gbSelectID) {
                    if (Alpha < 255) {
                        Alpha += 1;
                        GI.ZD_SetObjectAlpha(ID_OBJECT_THUMB, Alpha, GI.ZD_DRAW_REDRAW);
                    }
                }
                else {
                    if (Alpha > 0) {
                        Alpha -= 1;
                        GI.ZD_SetObjectAlpha(ID_OBJECT_THUMB, Alpha, GI.ZD_DRAW_REDRAW);
                    }
                }

                if (SWITCH_Marquee.SwitchStatus) DrawMarquee();

                // Update control display
                GI.UpdateWindow(hGDImageCtrl, false);
            }    
        }

        private void DrawMarquee()
        {
            int X, Y;
            GI.ZD_GetObjectXY(ID_OBJECT_TEXT, out X, out Y);
            X -= 2; if (X < -gnMarqueeWidth) X = GDImage_Template.Size.Width;
            GI.ZD_SetObjectXY(ID_OBJECT_TEXT, X, Y, GI.ZD_DRAW_DEFERRED);
        }

        private void Wallpaper_Click()
        {
            if (SWITCH_Wallpaper.SwitchStatus)
            {
                GI.ZI_SetFromFile(hGDImageCtrl, Api.ExePath() + "Back.jpg");
            }
            else
            {
                GI.ZI_SetFromFile(hGDImageCtrl, Api.ExePath() + "NoBackground.gif");
            }
        }

        //private void SaveInitialValue()
        //{
        //    Focal_InitialValue = POT_Focal.Value;
        //    Speed_InitialValue = POT_Speed.Value;
        //    Y_InitialValue = POT_Y.Value;
        //}

        private void BTN_Reset_Click(object sender, EventArgs e)
        {
            int UboundSpriteData = SpriteData.Length;
            int yOffset, Item;

            if (SWITCH_Mirror.SwitchStatus) {
                yOffset = 8; }
            else {
                yOffset = 0;
            }

            for (int K = 0; K < UboundSpriteData; K++)
            {
                Item = ID_OBJECT_SPRITE + K + 1;
                GI.ZD_SetObjectXY(Item, SpriteData[K].xPos, SpriteData[K].yPos - yOffset, GI.ZD_DRAW_DEFERRED);
                GI.ZD_SetObjectAngle(Item, 0, GI.ZD_DRAW_DEFERRED);
            }

            // Update control display
            GI.UpdateWindow(hGDImageCtrl, false);

        }


    } // END class MAIN_Form

} // END namespace

